#!/usr/bin/env python3
# coding=utf-8

__version__ = "0.0.1a"
__license__ = "GPLv3"
__description__ = "TUI client for twitch with thumbnails"
__url_repository__ = "https://github.com/WANDEX/twitchez"
__url_bug_reports__ = "https://github.com/WANDEX/twitchez/issues"
__url_project__ = __url_repository__
__author__ = "WANDEX"
